@@ -1,42 +1,37 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 /**
  * @var CView $this
  */
-?>
 
+?>
 window.host_edit_popup = {
 	overlay: null,
 	dialogue: null,
 	form: null,
 
-	init({popup_url, form_name, host_interfaces, host_is_discovered, warnings}) {
+	init({popup_url, form_name, host_interfaces, proxy_groupid, host_is_discovered, warnings}) {
 		this.overlay = overlays_stack.getById('host_edit');
 		this.dialogue = this.overlay.$dialogue[0];
 		this.form = this.overlay.$dialogue.$body[0].querySelector('form');
 
 		history.replaceState({}, '', popup_url);
 
-		host_edit.init({form_name, host_interfaces, host_is_discovered});
+		host_edit.init({form_name, host_interfaces, proxy_groupid, host_is_discovered});
 
 		if (warnings.length) {
 			const message_box = warnings.length == 1
@@ -47,6 +42,92 @@ window.host_edit_popup = {
 
 			this.form.parentNode.insertBefore(message_box, this.form);
 		}
+
+		this.initial_form_fields = getFormFields(this.form);
+		this.initEvents();
+	},
+
+	initEvents() {
+		this.form.addEventListener('click', (e) => {
+			const target = e.target;
+
+			if (target.classList.contains('js-edit-linked-template')) {
+				this.editTemplate({templateid: e.target.dataset.templateid});
+			}
+			else if (target.classList.contains('js-edit-proxy')) {
+				this.editProxy(e.target.dataset.proxyid);
+			}
+			else if (target.classList.contains('js-update-item')) {
+				this.editItem(target, target.dataset);
+			}
+		});
+	},
+
+	editTemplate(parameters) {
+		if (!this.isConfirmed()) {
+			return;
+		}
+
+		overlayDialogueDestroy(this.overlay.dialogueid);
+
+		const overlay = PopUp('template.edit', parameters, {
+			dialogueid: 'templates-form',
+			dialogue_class: 'modal-popup-large',
+			prevent_navigation: true
+		});
+
+		overlay.$dialogue[0].addEventListener('dialogue.submit', (e) =>
+			this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: e.detail}))
+		);
+	},
+
+	editProxy(proxyid) {
+		if (!this.isConfirmed()) {
+			return;
+		}
+
+		overlayDialogueDestroy(this.overlay.dialogueid);
+
+		const overlay = PopUp('popup.proxy.edit', {proxyid}, {
+			dialogueid: 'proxy_edit',
+			dialogue_class: 'modal-popup-static',
+			prevent_navigation: true
+		});
+
+		overlay.$dialogue[0].addEventListener('dialogue.submit', (e) =>
+			this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: e.detail}))
+		);
+	},
+
+	editItem(target, data) {
+		if (!this.isConfirmed()) {
+			return;
+		}
+
+		overlayDialogueDestroy(this.overlay.dialogueid);
+
+		const overlay = PopUp('item.edit', data, {
+			dialogueid: 'item-edit',
+			dialogue_class: 'modal-popup-large',
+			trigger_element: target
+		});
+
+		overlay.$dialogue[0].addEventListener('dialogue.submit', (e) =>
+			this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: e.detail})),
+			{once: true}
+		);
+	},
+
+	isConfirmed() {
+		const form_fields = getFormFields(this.form);
+
+		if (JSON.stringify(this.initial_form_fields) !== JSON.stringify(form_fields)) {
+			if (!window.confirm(<?= json_encode(_('Any changes made in the current form will be lost.')) ?>)) {
+				return false;
+			}
+		}
+
+		return true;
 	},
 
 	submit() {
@@ -69,14 +150,14 @@ window.host_edit_popup = {
 				overlayDialogueDestroy(this.overlay.dialogueid);
 
 				if ('hostid' in fields) {
-					this.dialogue.dispatchEvent(new CustomEvent('dialogue.update', {
+					this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {
 						detail: {
 							success: response.success
 						}
 					}));
 				}
 				else {
-					this.dialogue.dispatchEvent(new CustomEvent('dialogue.create', {
+					this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {
 						detail: {
 							success: response.success
 						}
@@ -102,25 +183,12 @@ window.host_edit_popup = {
 		});
 	},
 
-	fullClone() {
-		this.overlay.setLoading();
-		const parameters = host_edit.preprocessFormFields(getFormFields(this.form), true);
-		delete parameters.sid;
-		parameters.full_clone = 1;
-
-		PopUp('popup.host.edit', parameters, {
-			dialogueid: 'host_edit',
-			dialogue_class: 'modal-popup-large',
-			prevent_navigation: true
-		});
-	},
-
 	delete(hostid) {
 		this.removePopupMessages();
 
 		const curl = new Curl('zabbix.php');
 		curl.setArgument('action', 'host.massdelete');
-		curl.setArgument('<?= CCsrfTokenHelper::CSRF_TOKEN_NAME ?>', <?= json_encode(CCsrfTokenHelper::get('host')) ?>);
+		curl.setArgument(CSRF_TOKEN_NAME, <?= json_encode(CCsrfTokenHelper::get('host')) ?>);
 
 		fetch(curl.getUrl(), {
 			method: 'POST',
@@ -135,11 +203,7 @@ window.host_edit_popup = {
 
 				overlayDialogueDestroy(this.overlay.dialogueid);
 
-				this.dialogue.dispatchEvent(new CustomEvent('dialogue.delete', {
-					detail: {
-						success: response.success
-					}
-				}));
+				this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: response}));
 			})
 			.catch(this.ajaxExceptionHandler)
 			.finally(() => {
