@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -27,19 +22,25 @@ class DB {
 	const RESERVEIDS_ERROR = 2;
 	const SCHEMA_ERROR = 3;
 	const INPUT_ERROR = 4;
+	const INIT_ERROR = 5;
 
 	const TABLE_TYPE_CONFIG = 1;
 	const TABLE_TYPE_HISTORY = 2;
 
-	const FIELD_TYPE_INT = 'int';
-	const FIELD_TYPE_CHAR = 'char';
-	const FIELD_TYPE_ID = 'id';
-	const FIELD_TYPE_FLOAT = 'float';
-	const FIELD_TYPE_UINT = 'uint';
-	const FIELD_TYPE_BLOB = 'blob';
-	const FIELD_TYPE_TEXT = 'text';
-	const FIELD_TYPE_NCLOB = 'nclob';
-	const FIELD_TYPE_CUID = 'cuid';
+	const FIELD_TYPE_INT = 0x0001;
+	const FIELD_TYPE_CHAR = 0x0002;
+	const FIELD_TYPE_ID = 0x0004;
+	const FIELD_TYPE_FLOAT = 0x0008;
+	const FIELD_TYPE_UINT = 0x0010;
+	const FIELD_TYPE_BLOB = 0x0020;
+	const FIELD_TYPE_TEXT = 0x0040;
+	const FIELD_TYPE_CUID = 0x0080;
+	const FIELD_TYPE_NCLOB = 0x0100;
+
+	const SUPPORTED_FILTER_TYPES = self::FIELD_TYPE_INT | self::FIELD_TYPE_CHAR | self::FIELD_TYPE_ID |
+		self::FIELD_TYPE_FLOAT | self::FIELD_TYPE_UINT | self::FIELD_TYPE_CUID;
+	const SUPPORTED_SEARCH_TYPES = self::FIELD_TYPE_CHAR | self::FIELD_TYPE_TEXT | self::FIELD_TYPE_CUID |
+		self::FIELD_TYPE_NCLOB;
 
 	private static $schema = null;
 
@@ -78,16 +79,14 @@ class DB {
 	}
 
 	/**
-	 * Reserve ids for primary key of passed table.
-	 * If record for table does not exist or value is out of range, ids record is created
-	 * using maximum id from table or minimum allowed value.
-	 *
-	 * @throw APIException
-	 *
-	 * @static
+	 * Reserve IDs for primary key of passed table.
+	 * If record for table does not exist or value is out of range, ids record is created using maximum ID from table
+	 * or minimum allowed value.
 	 *
 	 * @param string $table table name
-	 * @param int $count number of ids to reserve
+	 * @param int $count number of IDs to reserve
+	 *
+	 * @throws APIException
 	 *
 	 * @return string
 	 */
@@ -144,19 +143,17 @@ class DB {
 	}
 
 	/**
-	 * Refresh id record for given table.
-	 * Record is deleted and then created again with value of maximum id from table or minimum allowed.
-	 *
-	 * @throw APIException
-	 *
-	 * @static
+	 * Refresh ID record for given table.
+	 * Record is deleted and then created again with value of maximum ID from table or minimum allowed.
 	 *
 	 * @param string $table table name
-	 * @param int    $count number of ids to reserve
+	 * @param int    $count number of IDs to reserve
+	 *
+	 * @throws APIException
 	 *
 	 * @return string
 	 */
-	private static function refreshIds($table, $count) {
+	public static function refreshIds($table, $count) {
 		$tableSchema = self::getSchema($table);
 		$id_name = $tableSchema['key'];
 
@@ -192,40 +189,57 @@ class DB {
 	}
 
 	/**
-	 * Returns the array describing the database schema.
+	 * Returns an array describing the database schema.
 	 *
 	 * If the $table parameter is passed, the method will return the schema for the given table,
 	 * otherwise - for the whole database.
 	 *
-	 * @static
+	 * @param string|null $table
 	 *
-	 * @throws APIException if the given table does not exist
-	 *
-	 * @param string $table
+	 * @throws DBException if the given table does not exist.
 	 *
 	 * @return array
 	 */
-	public static function getSchema($table = null) {
-		if (is_null(self::$schema)) {
-			self::$schema = include(dirname(__FILE__).'/../../'.self::SCHEMA_FILE);
+	public static function getSchema(?string $table = null): array {
+		if (self::$schema === null) {
+			$schema = include __DIR__.'/../../'.self::SCHEMA_FILE;
+
+			global $DB;
+
+			if ($DB['TYPE'] === ZBX_DB_ORACLE) {
+				$config = DBfetch(DBselect('SELECT dbversion_status FROM config'));
+				$dbversion_status = $config ? (array) json_decode($config['dbversion_status'], true) : [];
+
+				foreach ($dbversion_status as $dbversion) {
+					if (array_key_exists('schema_diff', $dbversion)
+							&& array_key_exists('tables', $dbversion['schema_diff'])) {
+						foreach ($dbversion['schema_diff']['tables'] as $table_name => $table_params) {
+							foreach ($table_params['fields'] as $field_name => $field) {
+								$schema[$table_name]['fields'][$field_name]['type'] = $field['type'];
+								$schema[$table_name]['fields'][$field_name]['length'] = $field['length'];
+							}
+						}
+					}
+				}
+			}
+
+			self::$schema = $schema;
 		}
 
-		if (is_null($table)) {
+		if ($table === null) {
 			return self::$schema;
 		}
-		elseif (isset(self::$schema[$table])) {
-			return self::$schema[$table];
-		}
-		else {
+
+		if (!array_key_exists($table, self::$schema)) {
 			self::exception(self::SCHEMA_ERROR, _s('Table "%1$s" does not exist.', $table));
 		}
+
+		return self::$schema[$table];
 	}
 
 	/**
 	 * Returns the names of the fields that are used as the primary key of the table.
 	 *
-	 * @static
-	 *
 	 * @param string $table_name
 	 *
 	 * @return string
@@ -239,8 +253,6 @@ class DB {
 	/**
 	 * Returns true if the table $tableName has the $fieldName field.
 	 *
-	 * @static
-	 *
 	 * @param string $tableName
 	 * @param string $fieldName
 	 *
@@ -255,8 +267,6 @@ class DB {
 	/**
 	 * Returns length of the field.
 	 *
-	 * @static
-	 *
 	 * @param string $table_name
 	 * @param string $field_name
 	 *
@@ -267,11 +277,11 @@ class DB {
 
 		$schema = self::getSchema($table_name);
 
-		if ($schema['fields'][$field_name]['type'] == self::FIELD_TYPE_TEXT) {
+		if ($schema['fields'][$field_name]['type'] & self::FIELD_TYPE_TEXT) {
 			return ($DB['TYPE'] == ZBX_DB_ORACLE) ? 2048 : 65535;
 		}
 
-		if ($schema['fields'][$field_name]['type'] == self::FIELD_TYPE_NCLOB) {
+		if ($schema['fields'][$field_name]['type'] & self::FIELD_TYPE_NCLOB) {
 			return 65535;
 		}
 
@@ -318,7 +328,7 @@ class DB {
 		$updated_values = [];
 
 		// Discard field names not existing in the target table.
-		$fields = array_intersect_key(DB::getSchema($table_name)['fields'], $new_values);
+		$fields = array_intersect_key(self::getSchema($table_name)['fields'], $new_values);
 
 		foreach ($fields as $name => $spec) {
 			if (!array_key_exists($name, $old_values)) {
@@ -326,26 +336,18 @@ class DB {
 				continue;
 			}
 
-			switch ($spec['type']) {
-				case DB::FIELD_TYPE_ID:
-					if (bccomp($new_values[$name], $old_values[$name]) != 0) {
-						$updated_values[$name] = $new_values[$name];
-					}
-					break;
-
-				case DB::FIELD_TYPE_INT:
-				case DB::FIELD_TYPE_UINT:
-				case DB::FIELD_TYPE_FLOAT:
-					if ($new_values[$name] != $old_values[$name]) {
-						$updated_values[$name] = $new_values[$name];
-					}
-					break;
-
-				default:
-					if ($new_values[$name] !== $old_values[$name]) {
-						$updated_values[$name] = $new_values[$name];
-					}
-					break;
+			if ($spec['type'] & self::FIELD_TYPE_ID) {
+				if (bccomp($new_values[$name], $old_values[$name]) != 0) {
+					$updated_values[$name] = $new_values[$name];
+				}
+			}
+			elseif ($spec['type'] & (self::FIELD_TYPE_INT | self::FIELD_TYPE_UINT | self::FIELD_TYPE_FLOAT)) {
+				if ($new_values[$name] != $old_values[$name]) {
+					$updated_values[$name] = $new_values[$name];
+				}
+			}
+			elseif ($new_values[$name] !== $old_values[$name]) {
+				$updated_values[$name] = $new_values[$name];
 			}
 		}
 
@@ -381,77 +383,68 @@ class DB {
 				}
 			}
 			else {
-				switch ($tableSchema['fields'][$field]['type']) {
-					case self::FIELD_TYPE_CUID:
-					case self::FIELD_TYPE_CHAR:
+				if ($tableSchema['fields'][$field]['type'] & (self::FIELD_TYPE_CUID | self::FIELD_TYPE_CHAR)) {
+					$length = mb_strlen($values[$field]);
+
+					if ($length > $tableSchema['fields'][$field]['length']) {
+						self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is %4$d characters.',
+							$values[$field], $field, $length, $tableSchema['fields'][$field]['length']));
+					}
+					$values[$field] = zbx_dbstr($values[$field]);
+				}
+				elseif ($tableSchema['fields'][$field]['type'] & (self::FIELD_TYPE_ID | self::FIELD_TYPE_UINT)) {
+					if (!zbx_ctype_digit($values[$field])) {
+						self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for unsigned int field "%2$s".', $values[$field], $field));
+					}
+					$values[$field] = zbx_dbstr($values[$field]);
+				}
+				elseif ($tableSchema['fields'][$field]['type'] & self::FIELD_TYPE_INT) {
+					if (!zbx_is_int($values[$field])) {
+						self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for int field "%2$s".', $values[$field], $field));
+					}
+					$values[$field] = zbx_dbstr($values[$field]);
+				}
+				elseif ($tableSchema['fields'][$field]['type'] & self::FIELD_TYPE_FLOAT) {
+					if (!is_numeric($values[$field])) {
+						self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for float field "%2$s".', $values[$field], $field));
+					}
+					$values[$field] = zbx_dbstr($values[$field]);
+				}
+				elseif ($tableSchema['fields'][$field]['type'] & self::FIELD_TYPE_TEXT) {
+					if ($DB['TYPE'] == ZBX_DB_ORACLE) {
 						$length = mb_strlen($values[$field]);
 
-						if ($length > $tableSchema['fields'][$field]['length']) {
+						if ($length > 2048) {
 							self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is %4$d characters.',
-								$values[$field], $field, $length, $tableSchema['fields'][$field]['length']));
+								$values[$field], $field, $length, 2048));
 						}
+					}
+					$values[$field] = zbx_dbstr($values[$field]);
+				}
+				elseif ($tableSchema['fields'][$field]['type'] & self::FIELD_TYPE_NCLOB) {
+					// Using strlen because 4000 bytes is largest possible string literal in oracle query.
+					if ($DB['TYPE'] == ZBX_DB_ORACLE && strlen($values[$field]) > ORACLE_MAX_STRING_SIZE) {
+						$chunks = zbx_dbstr(self::chunkMultibyteStr($values[$field], ORACLE_MAX_STRING_SIZE));
+						$values[$field] = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
+					}
+					else {
 						$values[$field] = zbx_dbstr($values[$field]);
-						break;
-
-					case self::FIELD_TYPE_ID:
-					case self::FIELD_TYPE_UINT:
-						if (!zbx_ctype_digit($values[$field])) {
-							self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for unsigned int field "%2$s".', $values[$field], $field));
-						}
-						$values[$field] = zbx_dbstr($values[$field]);
-						break;
-
-					case self::FIELD_TYPE_INT:
-						if (!zbx_is_int($values[$field])) {
-							self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for int field "%2$s".', $values[$field], $field));
-						}
-						$values[$field] = zbx_dbstr($values[$field]);
-						break;
-
-					case self::FIELD_TYPE_FLOAT:
-						if (!is_numeric($values[$field])) {
-							self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "%1$s" for float field "%2$s".', $values[$field], $field));
-						}
-						$values[$field] = zbx_dbstr($values[$field]);
-						break;
-
-					case self::FIELD_TYPE_TEXT:
-						if ($DB['TYPE'] == ZBX_DB_ORACLE) {
-							$length = mb_strlen($values[$field]);
-
-							if ($length > 2048) {
-								self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is %4$d characters.',
-									$values[$field], $field, $length, 2048));
-							}
-						}
-						$values[$field] = zbx_dbstr($values[$field]);
-						break;
-
-					case self::FIELD_TYPE_NCLOB:
-						// Using strlen because 4000 bytes is largest possible string literal in oracle query.
-						if ($DB['TYPE'] == ZBX_DB_ORACLE && strlen($values[$field]) > ORACLE_MAX_STRING_SIZE) {
-							$chunks = zbx_dbstr(self::chunkMultibyteStr($values[$field], ORACLE_MAX_STRING_SIZE));
-							$values[$field] = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
-						}
-						else {
+					}
+				}
+				elseif ($tableSchema['fields'][$field]['type'] & self::FIELD_TYPE_BLOB) {
+					switch ($DB['TYPE']) {
+						case ZBX_DB_MYSQL:
 							$values[$field] = zbx_dbstr($values[$field]);
-						}
-						break;
+							break;
 
-					case self::FIELD_TYPE_BLOB:
-						switch ($DB['TYPE']) {
-							case ZBX_DB_MYSQL:
-								$values[$field] = zbx_dbstr($values[$field]);
-								break;
+						case ZBX_DB_POSTGRESQL:
+							$values[$field] = "'".pg_escape_bytea($DB['DB'], $values[$field])."'";
+							break;
 
-							case ZBX_DB_POSTGRESQL:
-								$values[$field] = "'".pg_escape_bytea($DB['DB'], $values[$field])."'";
-								break;
-
-							case ZBX_DB_ORACLE:
-								// Do nothing; Check CImage.php to see how to update BLOB data with ORACLE DB.
-								break;
-						}
+						case ZBX_DB_ORACLE:
+							// Do nothing; Check CImage.php to see how to update BLOB data with ORACLE DB.
+							break;
+					}
 				}
 			}
 		}
@@ -480,8 +473,6 @@ class DB {
 	/**
 	 * Returns the records that match the given criteria.
 	 *
-	 * @static
-	 *
 	 * @param string $tableName
 	 * @param array $criteria   An associative array of field-value pairs, where value can be either a single value
 	 *                          or an array (IN)
@@ -541,8 +532,6 @@ class DB {
 	/**
 	 * Returns the list of mandatory fields with default values for INSERT statements.
 	 *
-	 * @static
-	 *
 	 * @param array $table_schema
 	 *
 	 * @return array
@@ -555,7 +544,7 @@ class DB {
 		switch ($DB['TYPE']) {
 			case ZBX_DB_MYSQL:
 				foreach ($table_schema['fields'] as $name => $field) {
-					if ($field['type'] == self::FIELD_TYPE_TEXT || $field['type'] == self::FIELD_TYPE_NCLOB) {
+					if ($field['type'] & self::FIELD_TYPE_TEXT || $field['type'] & self::FIELD_TYPE_NCLOB) {
 						$mandatory_fields += [$name => $field['default']];
 					}
 				}
@@ -563,7 +552,7 @@ class DB {
 
 			case ZBX_DB_ORACLE:
 				foreach ($table_schema['fields'] as $name => $field) {
-					if ($field['type'] == self::FIELD_TYPE_BLOB) {
+					if ($field['type'] & self::FIELD_TYPE_BLOB) {
 						$mandatory_fields += [$name => 'EMPTY_BLOB()'];
 					}
 				}
@@ -584,23 +573,20 @@ class DB {
 		$table_schema = self::getSchema($table);
 		$resultids = [];
 
-		if ($table_schema['fields'][$table_schema['key']]['type'] === DB::FIELD_TYPE_ID) {
+		if ($table_schema['fields'][$table_schema['key']]['type'] & self::FIELD_TYPE_ID) {
 			$id = self::reserveIds($table, count($values));
 		}
 
 		foreach ($values as $key => &$row) {
-			switch ($table_schema['fields'][$table_schema['key']]['type']) {
-				case DB::FIELD_TYPE_ID:
-					$resultids[$key] = $id;
-					$row = [$table_schema['key'] => $id] + $row;
-					$id = bcadd($id, 1, 0);
-					break;
-
-				case DB::FIELD_TYPE_CUID:
-					$id = CCuid::generate();
-					$resultids[$key] = $id;
-					$row = [$table_schema['key'] => $id] + $row;
-					break;
+			if ($table_schema['fields'][$table_schema['key']]['type'] & self::FIELD_TYPE_ID) {
+				$resultids[$key] = $id;
+				$row = [$table_schema['key'] => $id] + $row;
+				$id = bcadd($id, 1, 0);
+			}
+			elseif ($table_schema['fields'][$table_schema['key']]['type'] & self::FIELD_TYPE_CUID) {
+				$id = CCuid::generate();
+				$resultids[$key] = $id;
+				$row = [$table_schema['key'] => $id] + $row;
 			}
 		}
 		unset($row);
@@ -634,6 +620,7 @@ class DB {
 			$row += $mandatory_fields;
 
 			self::checkValueTypes($table_schema, $row);
+			self::uppercaseValues($table, $row);
 		}
 		unset($row);
 
@@ -671,6 +658,8 @@ class DB {
 				self::exception(self::DBEXECUTE_ERROR, _s('Cannot perform update statement on table "%1$s" without values.', $table));
 			}
 
+			self::uppercaseValues($table, $row['values']);
+
 			// set creation
 			$sqlSet = '';
 			foreach ($row['values'] as $field => $value) {
@@ -709,11 +698,9 @@ class DB {
 	/**
 	 * Updates the values by the given PK.
 	 *
-	 * @static
-	 *
 	 * @param string $tableName
 	 * @param string $pk
-	 * @param array $values
+	 * @param array  $values
 	 *
 	 * @return bool
 	 */
@@ -728,8 +715,6 @@ class DB {
 	 * Saves the given records to the database. If the record has the primary key set, it is updated, otherwise - a new
 	 * record is inserted. For new records the newly generated PK is added to the result.
 	 *
-	 * @static
-	 *
 	 * @param $tableName
 	 * @param $data
 	 *
@@ -770,11 +755,9 @@ class DB {
 	 *
 	 * All of the records must have the primary key defined.
 	 *
-	 * @static
-	 *
-	 * @param $tableName
-	 * @param array $oldRecords
-	 * @param array $newRecords
+	 * @param string $tableName
+	 * @param array  $oldRecords
+	 * @param array  $newRecords
 	 *
 	 * @return array    the new records, that have been passed with the primary keys set for newly inserted records
 	 */
@@ -807,7 +790,7 @@ class DB {
 
 		// delete remaining records
 		if ($oldRecords) {
-			DB::delete($tableName, [
+			self::delete($tableName, [
 				$pk => array_keys($oldRecords)
 			]);
 		}
@@ -815,89 +798,12 @@ class DB {
 		return $newRecords;
 	}
 
-	/**
-	 * Replaces the records given in $groupedOldRecords with the ones given in $groupedNewRecords.
-	 *
-	 * This method can be used to replace related objects in one-to-many relations. Both old and new records
-	 * must be grouped by the ID of the record they belong to. The records will be matched by position, instead of
-	 * the primary key as in DB::replace(). That is, the first new record will update the first old one, second new
-	 * record - the second old one, etc. Since the records are matched by position, the new records should not contain
-	 * primary keys.
-	 *
-	 * Example 1:
-	 * $old = array(2 => array( array('gitemid' => 1, 'color' => 'FF0000') ));
-	 * $new = array(2 => array( array('color' => '00FF00') ));
-	 * var_dump(DB::replaceByPosition('items', $old, $new));
-	 * // array(array('gitemid' => 1, 'color' => '00FF00'))
-	 *
-	 * The new record updated the old one.
-	 *
-	 * Example 2:
-	 * $old = array(2 => array( array('gitemid' => 1, 'color' => 'FF0000') ));
-	 * $new = array(
-	 *     2 => array(
-	 *         array('color' => '00FF00'),
-	 *         array('color' => '0000FF')
-	 *     )
-	 * );
-	 * var_dump(DB::replaceByPosition('items', $old, $new));
-	 * // array(array('gitemid' => 1, 'color' => '00FF00'), array('gitemid' => 2, 'color' => '0000FF'))
-	 *
-	 * The first record was updated, the second one - created.
-	 *
-	 * Example 3:
-	 * $old = array(
-	 *     2 => array(
-	 *         array('gitemid' => 1, 'color' => 'FF0000'),
-	 *         array('gitemid' => 2, 'color' => '0000FF')
-	 *     )
-	 * );
-	 * $new = array(2 => array( array('color' => '00FF00') ));
-	 * var_dump(DB::replaceByPosition('items', $old, $new));
-	 * // array(array('gitemid' => 1, 'color' => '00FF00'))
-	 *
-	 * The first record was updated, the second one - deleted.
-	 *
-	 * @param string 	$tableName			table to update
-	 * @param array 	$groupedOldRecords	grouped old records
-	 * @param array 	$groupedNewRecords	grouped new records
-	 *
-	 * @return array	array of new records not grouped (!).
-	 */
-	public static function replaceByPosition($tableName, array $groupedOldRecords, array $groupedNewRecords) {
-		$pk = self::getPk($tableName);
-
-		$allOldRecords = [];
-		$allNewRecords = [];
-		foreach ($groupedNewRecords as $key => $newRecords) {
-			// if records exist for the parent object - replace them, otherwise create new records
-			if (isset($groupedOldRecords[$key])) {
-				$oldRecords = $groupedOldRecords[$key];
-
-				// updated the records by position
-				$newRecords = self::mergeRecords($oldRecords, $newRecords, $pk);
-
-				foreach ($oldRecords as $record) {
-					$allOldRecords[] = $record;
-				}
-			}
-
-			foreach ($newRecords as $record) {
-				$allNewRecords[] = $record;
-			}
-		}
-
-		// replace the old records with the new ones
-		return self::replace($tableName, $allOldRecords, $allNewRecords);
-	}
-
 	/**
 	 * Compares the fields, that are present in both records, and returns true if any of the values differ.
 	 *
-	 * @static
-	 * @param $tableName
-	 * @param array $oldRecord
-	 * @param array $newRecord
+	 * @param string $tableName
+	 * @param array  $oldRecord
+	 * @param array  $newRecord
 	 *
 	 * @return bool
 	 */
@@ -913,30 +819,6 @@ class DB {
 		return false;
 	}
 
-	/**
-	 * Replace each record in $oldRecords with a corresponding record in $newRecords, but keep the old record IDs.
-	 * The records are match by position, that is, the first new record, replaces the first old record and etc.
-	 * If there are less $newRecords than $oldRecords, the remaining old records will be discarded.
-	 *
-	 * @param array 	$oldRecords		array of old records
-	 * @param array 	$newRecords		array of new records
-	 * @param string 	$pk				name of the private key column
-	 *
-	 * @return array	array of new records with the primary keys from the old ones
-	 */
-	protected static function mergeRecords(array $oldRecords, array $newRecords, $pk) {
-		$result = [];
-		foreach ($newRecords as $i => $record) {
-			if (isset($oldRecords[$i])) {
-				$record[$pk] = $oldRecords[$i][$pk];
-			}
-
-			$result[] = $record;
-		}
-
-		return $result;
-	}
-
 	/**
 	 * Delete data from DB.
 	 *
@@ -1075,6 +957,43 @@ class DB {
 		return ($table_alias !== null) ? $table_alias.'.'.$field_name : $field_name;
 	}
 
+	/**
+	 * Convert field to uppercase or substitute it with its pre-upcased variant.
+	 *
+	 * @param string      $field_name
+	 * @param string      $table_name
+	 * @param string|null $table_alias
+	 *
+	 * @return string
+	 */
+	public static function uppercaseField(string $field_name, string $table_name, string $table_alias = null): string {
+		if ($table_alias === null) {
+			$table_alias = $table_name;
+		}
+
+		if ($field_name === 'name' && self::hasField($table_name, 'name_upper')) {
+			return $table_alias.'.name_upper';
+		}
+
+		if ($field_name === 'name_resolved' && self::hasField($table_name, 'name_resolved_upper')) {
+			return $table_alias.'.name_resolved_upper';
+		}
+
+		return 'UPPER('.$table_alias.'.'.$field_name.')';
+	}
+
+	/**
+	 * Convert field values to uppercase.
+	 *
+	 * @param string $table_name
+	 * @param array  $row
+	 */
+	public static function uppercaseValues(string $table_name, array &$row): void {
+		if (array_key_exists('name_resolved', $row) && self::hasField($table_name, 'name_resolved_upper')) {
+			$row['name_resolved_upper'] = 'UPPER('.$row['name_resolved'].')';
+		}
+	}
+
 	/**
 	 * Builds an SQL parts array from the given options.
 	 *
@@ -1169,18 +1088,14 @@ class DB {
 			$field_schema = $table_schema['fields'][$pk];
 			$field_name = self::fieldId($pk, $table_alias);
 
-			switch ($field_schema['type']) {
-				case self::FIELD_TYPE_ID:
-					$sql_parts['where'][] = dbConditionId($field_name, $options[$pk_option]);
-					break;
-
-				case self::FIELD_TYPE_INT:
-				case self::FIELD_TYPE_UINT:
-					$sql_parts['where'][] = dbConditionInt($field_name, $options[$pk_option]);
-					break;
-
-				default:
-					$sql_parts['where'][] = dbConditionString($field_name, $options[$pk_option]);
+			if ($field_schema['type'] & self::FIELD_TYPE_ID) {
+				$sql_parts['where'][] = dbConditionId($field_name, $options[$pk_option]);
+			}
+			elseif ($field_schema['type'] & (self::FIELD_TYPE_INT | self::FIELD_TYPE_UINT)) {
+				$sql_parts['where'][] = dbConditionInt($field_name, $options[$pk_option]);
+			}
+			else {
+				$sql_parts['where'][] = dbConditionString($field_name, $options[$pk_option]);
 			}
 		}
 
@@ -1210,10 +1125,7 @@ class DB {
 	private static function applyQuerySearchOptions($table_name, array $options, $table_alias, array $sql_parts) {
 		global $DB;
 
-		$table_schema = DB::getSchema($table_name);
-		$unsupported_types = [self::FIELD_TYPE_INT, self::FIELD_TYPE_ID, self::FIELD_TYPE_FLOAT, self::FIELD_TYPE_UINT,
-			self::FIELD_TYPE_BLOB
-		];
+		$table_schema = self::getSchema($table_name);
 
 		$start = $options['startSearch'] ? '' : '%';
 		$glue = $options['searchByAny'] ? ' OR ' : ' AND ';
@@ -1229,7 +1141,7 @@ class DB {
 
 			$field_schema = $table_schema['fields'][$field_name];
 
-			if (in_array($field_schema['type'], $unsupported_types)) {
+			if (($field_schema['type'] & self::SUPPORTED_SEARCH_TYPES) == 0) {
 				self::exception(self::SCHEMA_ERROR,
 					vsprintf('%s: field "%s.%s" has an unsupported type.', [__FUNCTION__, $table_name, $field_name])
 				);
@@ -1244,16 +1156,16 @@ class DB {
 				$pattern = mb_strtoupper(strtr($pattern, ['!' => '!!', '%' => '!%', '_' => '!_']));
 				$pattern = $start.$pattern.'%';
 
-				if ($DB['TYPE'] == ZBX_DB_ORACLE && $field_schema['type'] === DB::FIELD_TYPE_NCLOB
+				if ($DB['TYPE'] == ZBX_DB_ORACLE && $field_schema['type'] & self::FIELD_TYPE_NCLOB
 						&& strlen($pattern) > ORACLE_MAX_STRING_SIZE) {
-					$chunks = zbx_dbstr(DB::chunkMultibyteStr($pattern, ORACLE_MAX_STRING_SIZE));
+					$chunks = zbx_dbstr(self::chunkMultibyteStr($pattern, ORACLE_MAX_STRING_SIZE));
 					$pattern = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
 				}
 				else {
 					$pattern = zbx_dbstr($pattern);
 				}
 
-				$search[] = 'UPPER('.self::fieldId($field_name, $table_alias).') LIKE '.$pattern." ESCAPE '!'";
+				$search[] = self::uppercaseField($field_name, $table_name, $table_alias).' LIKE '.$pattern." ESCAPE '!'";
 			}
 		}
 
@@ -1291,7 +1203,7 @@ class DB {
 
 			$field_schema = $table_schema['fields'][$field_name];
 
-			if ($field_schema['type'] == self::FIELD_TYPE_TEXT || $field_schema['type'] == self::FIELD_TYPE_NCLOB) {
+			if (($field_schema['type'] & self::SUPPORTED_FILTER_TYPES) == 0) {
 				self::exception(self::SCHEMA_ERROR,
 					vsprintf('%s: field "%s.%s" has an unsupported type.', [__FUNCTION__, $table_name, $field_name])
 				);
@@ -1305,18 +1217,14 @@ class DB {
 				$value = [$value];
 			}
 
-			switch ($field_schema['type']) {
-				case self::FIELD_TYPE_ID:
-					$filter[] = dbConditionId(self::fieldId($field_name, $table_alias), $value);
-					break;
-
-				case self::FIELD_TYPE_INT:
-				case self::FIELD_TYPE_UINT:
-					$filter[] = dbConditionInt(self::fieldId($field_name, $table_alias), $value);
-					break;
-
-				default:
-					$filter[] = dbConditionString(self::fieldId($field_name, $table_alias), $value);
+			if ($field_schema['type'] & self::FIELD_TYPE_ID) {
+				$filter[] = dbConditionId(self::fieldId($field_name, $table_alias), $value);
+			}
+			elseif ($field_schema['type'] & (self::FIELD_TYPE_INT | self::FIELD_TYPE_UINT)) {
+				$filter[] = dbConditionInt(self::fieldId($field_name, $table_alias), $value);
+			}
+			else {
+				$filter[] = dbConditionString(self::fieldId($field_name, $table_alias), $value);
 			}
 		}
 
