@@ -116,7 +116,9 @@ class CRouter {
 		'hintbox.eventlist'							=> ['CControllerHintboxEventlist',						'layout.json',			'hintbox.eventlist'],
 		'host.create'								=> ['CControllerHostCreate',							'layout.json',			null],
 		'host.dashboard.view'						=> ['CControllerHostDashboardView',						'layout.htmlpage',		'monitoring.host.dashboard.view'],
+		'host.disable'								=> ['CControllerHostDisable',							'layout.json',			null],
 		'host.edit'									=> ['CControllerHostEdit',								'layout.htmlpage',		'configuration.host.edit'],
+		'host.enable'								=> ['CControllerHostEnable',							'layout.json',			null],
 		'host.list'									=> ['CControllerHostList',								'layout.htmlpage',		'configuration.host.list'],
 		'host.massdelete'							=> ['CControllerHostMassDelete',						'layout.json',			null],
 		'host.update'								=> ['CControllerHostUpdate',							'layout.json',			null],
