@@ -1,27 +1,22 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 /**
  * Default value in seconds, for poller interval.
  */
-ZBX_Notifications.POLL_INTERVAL = 30;
+ZBX_Notifications.POLL_INTERVAL = 5;
 
 ZBX_Notifications.ALARM_SEVERITY_RESOLVED = -1;
 ZBX_Notifications.ALARM_INFINITE_SERVER = -1;
@@ -38,8 +33,8 @@ ZBX_Notifications.ALARM_ONCE_SERVER = 1;
  * these "actions" by passing the new received state value through a method prefixed with <consume> that will adjust
  * instance's internal state, that in turn can be dispatched as "action". Other methods prefixed with <handle> responds
  * to other events than localStorage change event - (poll, focus, timeout..) and still they would reuse <consume>
- * domain methods and issue an anction via <push> if needed and call to render method explicitly. The <handlePushed> is
- * not reused on the instance that produces the action. This is so to reduce complexity and increase maintainebility,
+ * domain methods and issue an action via <push> if needed and call to render method explicitly. The <handlePushed> is
+ * not reused on the instance that produces the action. This is so to reduce complexity and increase maintainability,
  * because when an action produces an action, logic diverges deep, instead <consume> various domain within logic
  * and call `render` once, then <push> into localStorage once.
  *
@@ -381,13 +376,13 @@ ZBX_Notifications.prototype.handlePushedActiveTabid = function(tabid) {
 /**
  * When active tab is unloaded, any sibling tab is set to become active. If single session, then we drop LS (privacy).
  * We cannot know if this unload will happen because of navigation, scripted reload or a tab was just closed.
- * Latter is always assumed, so when navigating active tab, focus is deligated onto to any tab if possible,
- * then this tab might reclaim focus again at construction if during during that time document has focus.
+ * Latter is always assumed, so when navigating active tab, focus is delegated onto to any tab if possible,
+ * then this tab might reclaim focus again at construction if during that time document has focus.
  * At slow connection during page navigation there will be another active tab polling for notifications (if multitab).
  * Here `tab` is referred as ZBX_Notifications instance and `focus` - whether instance is `active` (not focused).
  *
  * @param {ZBX_BrowseTab} removed_tab  Current tab instance.
- * @param {array} other_tabids  List of alive tab ids (wuthout current tabid).
+ * @param {array} other_tabids  List of alive tab ids (without current tabid).
  */
 ZBX_Notifications.prototype.handleTabBeforeUnload = function(removed_tab, other_tabids) {
 	if (this.active && other_tabids.length) {
@@ -396,7 +391,7 @@ ZBX_Notifications.prototype.handleTabBeforeUnload = function(removed_tab, other_
 
 		/*
 		 * Solves problem happening in case when navigating to another top level domain. Chrome dispatches 'focusin'
-		 * event right after beforeunload event. It is crucial to not to respond to that, otherwise nonexisting tab
+		 * event right after beforeunload event. It is crucial not to respond to that, otherwise nonexisting tab
 		 * becomes active.
 		 */
 		this.becomeActive = function() {};
@@ -415,12 +410,20 @@ ZBX_Notifications.prototype.handleTabFocusIn = function() {
 };
 
 /**
- * @param {MouseEvent} e
+ * Close the notification box.
  */
-ZBX_Notifications.prototype.handleCloseClicked = function(e) {
-	this.fetch('notifications.read', {ids: this.getEventIds()})
-		.catch(console.error)
-		.then(function(resp) {
+ZBX_Notifications.prototype.handleCloseClicked = function() {
+	const data = {ids: this.getEventIds()};
+
+	data[CSRF_TOKEN_NAME] = this._csrf_token;
+
+	this
+		.fetch('notifications.read', data)
+		.then((resp) => {
+			if ('error' in resp) {
+				throw {error: resp.error};
+			}
+
 			resp.ids.forEach(function(id) {
 				this.removeById(id);
 				this.debounceRender();
@@ -428,39 +431,107 @@ ZBX_Notifications.prototype.handleCloseClicked = function(e) {
 
 			this.alarm.reset();
 			this.pushUpdates();
-		}.bind(this));
+		})
+		.catch((exception) => {
+			if (typeof exception === 'object' && 'error' in exception) {
+				clearMessages();
+
+				const message_box = makeMessageBox('bad', exception.error.messages, exception.error.title);
+
+				addMessage(message_box);
+			}
+			else {
+				console.log('Could not read notifications:', exception);
+			}
+		});
 };
 
 /**
- * @param {MouseEvent} e
+ * Handles snooze button click event.
  */
-ZBX_Notifications.prototype.handleSnoozeClicked = function(e) {
+ZBX_Notifications.prototype.handleSnoozeClicked = function() {
 	if (this.alarm.isSnoozed(this._cached_list)) {
 		return;
 	}
 
-	this.collection.map(function(notif) {
-		notif.updateRaw({snoozed: true});
-	});
+	const latest_event = Math.max(...this.collection.getRawList().map(event => parseInt(event.eventid, 10)));
+	const data = {eventid: latest_event};
 
-	this.consumeList(this.collection.getRawList());
+	data[CSRF_TOKEN_NAME] = this._csrf_token;
 
-	this.pushUpdates();
-	this.render();
+	this
+		.fetch('notifications.snooze', data)
+		.then((response) => {
+			if ('error' in response) {
+				throw {error: response.error};
+			}
+
+			this._cached_user_settings.snoozed_eventid = response.snoozed_eventid;
+
+			this.collection.map(function(notification) {
+				notification.updateRaw({snoozed: notification._raw.eventid <= latest_event});
+			});
+
+			this.consumeList(this.collection.getRawList());
+			this.pushUpdates();
+			this.render();
+		})
+		.catch((exception) => {
+			clearMessages();
+
+			let title, messages;
+
+			if (typeof exception === 'object' && 'error' in exception) {
+				title = exception.error.title;
+				messages = exception.error.messages;
+			}
+			else {
+				messages = [t('Unexpected server error.')];
+			}
+
+			const message_box = makeMessageBox('bad', messages, title);
+
+			addMessage(message_box);
+		});
 };
 
 /**
- * @param {MouseEvent} e
+ * Mute messages in the notification box.
  */
-ZBX_Notifications.prototype.handleMuteClicked = function(e) {
-	this.fetch('notifications.mute', {muted: this.alarm.muted ? 0 : 1})
-		.catch(console.error)
-		.then(function(resp) {
+ZBX_Notifications.prototype.handleMuteClicked = function() {
+	const data = {muted: this.alarm.muted ? 0 : 1};
+
+	data[CSRF_TOKEN_NAME] = this._csrf_token;
+
+	this
+		.fetch('notifications.mute', data)
+		.then((resp) => {
+			if ('error' in resp) {
+				throw {error: resp.error};
+			}
+
 			this._cached_user_settings.muted = (resp.muted == 1);
 			this.alarm.consume({muted: this._cached_user_settings.muted});
 			this.pushUpdates();
 			this.render();
-		}.bind(this));
+		})
+		.catch((exception) => {
+			clearMessages();
+
+			let title, messages;
+
+			if (typeof exception === 'object' && 'error' in exception) {
+				title = exception.error.title;
+				messages = exception.error.messages;
+			}
+			else {
+				messages = [t('Unexpected server error.')];
+			}
+
+			const message_box = makeMessageBox('bad', messages, title);
+
+			addMessage(message_box);
+		});
 };
 
 /**
@@ -477,8 +548,13 @@ ZBX_Notifications.prototype.handleMainLoopResp = function(resp) {
 		return;
 	}
 
+	this.collection.map(function(notification) {
+		notification.updateRaw({snoozed: notification._raw.eventid <= resp.settings.snoozed_eventid});
+	});
+
 	this.consumeUserSettings(resp.settings);
 	this.consumeList(resp.notifications);
+	this._csrf_token = resp[CSRF_TOKEN_NAME];
 	this.render();
 
 	this.pushUpdates();
@@ -497,7 +573,9 @@ ZBX_Notifications.prototype.handleAlarmStateChanged = function(alarm_state) {
  * user configuration, the list state to be rendered, has been consumed by collection before.
  */
 ZBX_Notifications.prototype.renderCollection = function() {
-	this.collection.render(this._cached_user_settings.severity_styles, this.alarm);
+	this.collection.render(this._cached_user_settings.severity_styles, this.alarm, this._cached_user_settings.username,
+		this._cached_user_settings.muted
+	);
 };
 
 /**
@@ -522,7 +600,7 @@ ZBX_Notifications.prototype.renderAudio = function() {
 
 /**
  * @param {string} resource  A value for 'action' parameter.
- * @param {object} params    Form data to be send.
+ * @param {object} params    Form data to be sent.
  *
  * @return {Promise}
  */
@@ -552,9 +630,27 @@ ZBX_Notifications.prototype.mainLoop = function() {
 		return;
 	}
 
-	this.fetch('notifications.get', {known_eventids: this.getEventIds()})
-		.catch(console.error)
-		.then(this.handleMainLoopResp.bind(this));
+	this
+		.fetch('notifications.get', {known_eventids: this.getEventIds()})
+		.then((resp) => {
+			if ('error' in resp) {
+				throw {error: resp.error};
+			}
+
+			this.handleMainLoopResp(resp);
+		})
+		.catch((exception) => {
+			if (typeof exception === 'object' && 'error' in exception) {
+				clearMessages();
+
+				const message_box = makeMessageBox('bad', exception.error.messages, exception.error.title);
+
+				addMessage(message_box);
+			}
+			else {
+				console.log('Could not get notifications:', exception);
+			}
+		});
 };
 
 /**
@@ -769,10 +865,10 @@ ZBX_NotificationsAlarm.prototype.isPlayed = function() {
 /**
  * @param {array} list  List of raw notifications.
  *
- * @return {bool}
+ * @return {boolean}
  */
 ZBX_NotificationsAlarm.prototype.isSnoozed = function(list) {
-	for (var i = 0; i < list.length; i++) {
+	for (let i = 0; i < list.length; i++) {
 		if (!list[i].snoozed) {
 			return false;
 		}
@@ -938,7 +1034,9 @@ $(function() {
 		pos_side = 10,
 		side = 'right';
 
-	main.appendChild(ntf_node);
+	if (main !== null) {
+		main.appendChild(ntf_node);
+	}
 
 	if (ntf_pos !== null && 'top' in ntf_pos) {
 		side = ('right' in ntf_pos ? 'right' : ('left' in ntf_pos ? 'left' : null));
@@ -951,7 +1049,7 @@ $(function() {
 	ntf_node.style.top = pos_top + 'px';
 	ntf_node.style[side] = pos_side + 'px';
 
-	$(ntf_node).draggable({handle: '>.dashboard-widget-head',
+	$(ntf_node).draggable({handle: '>.overlay-dialogue-header',
 		start: function(event, ui) {
 			ui.helper.data('containment', {
 				min_top: -main.offsetTop,
